{% extends g.auth.template or 'core/auth/base.html' %}
{% import 'core/partial/login_field.html' as login_field %}

{% block auth_subtitle %}
    <div class="sub header">{{ _("Please enter your password for this account.") }}</div>
{% endblock %}

{% block auth_form_content %}
    {{ login_field.render_field(form.password, class="autofocus") }}
{% endblock %}

{% block auth_right_buttons %}
    {% if features.has_smtp and features.has_password_recovery %}
        <a type="button" class="ui button" href="{{ url_for('core.auth.password.forgotten') }}">
            {{ _("Forgotten password") }}
        </a>
    {% endif %}
    <button type="submit" class="ui primary button">{{ _("Sign in") }}</button>
{% endblock %}
