"""oidc client require_signed_request_object attribute.

Revision ID: 1745234836
Revises: 1743675164
Create Date: 2025-04-21 13:27:16.520455
"""

from collections.abc import Sequence

import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision: str = "1745234836"
down_revision: str | None = "1743675164"
branch_labels: str | Sequence[str] | None = ()
depends_on: str | Sequence[str] | None = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "client",
        sa.Column("require_signed_request_object", sa.Boolean(), nullable=True),
    )
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("client", "require_signed_request_object")
    # ### end Alembic commands ###
