"""Client 'jwk' has been renamed 'jwks'.

Revision ID: 1738929510
Revises: 1736443538
Create Date: 2025-02-07 12:58:30.855043
"""

from collections.abc import Sequence

import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision: str = "1738929510"
down_revision: str | None = "1736443538"
branch_labels: str | Sequence[str] | None = ()
depends_on: str | Sequence[str] | None = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("client", sa.Column("jwks", sa.String(), nullable=True))
    op.drop_column("client", "jwk")
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("client", sa.Column("jwk", sa.VARCHAR(), nullable=True))
    op.drop_column("client", "jwks")
    # ### end Alembic commands ###
