{# Macros for Flask flash message rendering #}
{% macro messages() %}
    {% with messages = get_flashed_messages(with_categories=true) %}
        {% for category, message in messages %}
            <div class="ui
                        {% if messages|len > 1 %}
                            {% if loop.first %}top {% elif loop.last %}bottom {% endif %}attached
                        {% endif %}
                        message
                        {{ category }}">
                {{ message }}
            </div>
        {% endfor %}
    {% endwith %}
{% endmacro %}
